#Code to get counts
#Code requires a SIMPLE (NO LOOPS NO MULTIPLE EDGES)  graph as an edgelist tab separeted with node indices starting from 1 (NOT ZERO).
#To call this code use
# Rscript egonet_subgraphs.R myargs_egos.txt
#myargs.txt contains four lines giving (1) the paths to the edgelist (2) the output file  (3) type of local neighbourhood either "node_egos" or "edge_egos" (EDGE EGOS NOT AVAILABLE AT THE MOMENT) and (4) the number of steps taken for the ego-networks.
#DEGREE ZERO NODES ARE NOT TAKEN INTO ACCOUNT IN THIS SCRIPT, RESULT ONLY FOR NODES AT LEAST ONE EDGE

##############################
#read_argments from 

args=(commandArgs(TRUE))
batch_args <- read.table(args[1],header = FALSE)


elistdir= as.character(batch_args[1,1])
writeto= as.character(batch_args[2,1])
localmethod= as.character(batch_args[3,1])
step= as.numeric(as.character(batch_args[4,1]))

require(igraph)
require(orca)
##############################
#elistdir="~/Desktop/aring_graph.txt"
#writeto="~/Desktop/example_subgraphs_aring.txt"


##############################///
#
orca_orbitcounts=function(e_list_num, subgraph_size="four"){
  #If graph has edges#it now starts with 1, required by orca
  e_list=e_list_num
  #If graph does not have edges
  if(length(e_list)<1){ 
    print("empty edgelist?");stop("empty edgelist?")
  }
  if(!is.integer(e_list_num)){
    e_list=matrix(ncol=2,nrow=nrow(e_list_num))
    e_list[,1]=as.integer(e_list_num[,1]); e_list[,2]=as.integer(e_list_num[,2])
  }
  rm(e_list_num)
  if(min(e_list) <1 | min(e_list) > 1 ){ print("Wrong indexing for orca. It has to start at 1 and indices must be integers.");stop("Wrong indexing for orca.  It has to start at 1 and indices must be integers.")}
  if(subgraph_size=="four")
    orbitcounts=count4(e_list) #needs integer matrix
  if(subgraph_size=="five")
    orbitcounts=count5(e_list) #needs integer matrix
  return(orbitcounts)
}
##########/////////

orca_subgraphcounts=function(e_list){
  orbits=orca_orbitcounts(e_list,"four")
  subgraph=list(1,2:3,4,5:6,7:8,9,10:12,13:14,15)
  if(nrow(orbits) <1 ){
    print("empty edgelist?"); stop("empty edgelist?")
  }
  if(nrow(orbits) >= 1){
    counts=sapply(subgraph,function(l){if(length(l)<2) return( orbits[,l] ); rowSums(orbits[,l])  })
  }
  colnames(counts)=c("edge","2s","tri","4l","3s","sq","triedg","sqdiag","4clique")
  return(counts)  
}

###################/////
counts_in_egonetworks=function(elist,step){
  #elist[1,1]=as.character(elist[1,1])
  graph=graph.edgelist(elist,directed = FALSE)
  rm(elist)
  nvs= vcount(graph)
  countmatrix=matrix(nrow=nvs,ncol=10)
  kk=1;while( kk <= nvs){
    ego=graph.neighborhood(graph = graph,order = step,nodes = kk)[[1]]
    ego_elist=get.edgelist(ego,names = FALSE)
    whatmin=min(ego_elist)
    if(whatmin!=1)  ego_elist=ego_elist-whatmin+1 #orca needs to start at one
    currentegocounts= orca_subgraphcounts(ego_elist)
    countmatrix[kk,]= c(vcount(ego), colSums(currentegocounts) /c(2,3,3,4,4,4,4,4,4))
    kk=kk+1
    rm(ego)
    #cat(kk,"-" )
  }
  #
  colnames(countmatrix)=c("nv_ego","edge","2s","tri","4l","3s","sq","triedg","sqdiag","4clique")
  return(countmatrix)
}

###################/////
e_list=read.table(file = elistdir,sep = "\t",)
e_list=as.matrix(e_list)

if(localmethod=="node_egos"){
  result= counts_in_egonetworks(e_list,step) 
}
#
if(localmethod=="edge_egos"){
  print("NOT READY YET")
  break
  result= counts_in_edge_egonetworks(e_list,step) 
}

#
#Write results
write.table(result,file=writeto,sep = "\t",row.names = FALSE,col.names = TRUE,quote=FALSE)


###OTHER INFORMATION##
#ABOUT COMMAND LINE CALL: TAKE THE FOLLOWING CODE AT THE BEGINING OF THE SCRIPT
#cmd <- paste(commandArgs(), collapse=" ")
#cat("How R was invoked:\n");
#cat(cmd, "\n")

####EXAMPLE
#e_list<-read.table("/Users/Shadowlight/Dropbox/Research/Present research/Joint_research/Anatol/network_examples/BA_2500_20_1")
#e_list<-read.table("/Users/Shadowlight/Dropbox/Research/Present research/Joint_research/Anatol/network_examples/test.txt")
#e_list=as.matrix(e_list)
#e_list[1,1]=as.character(e_list[1,1])
#require(igraph)
#g<- graph.edgelist(e_list,directed = FALSE)
#g<-erdos.renyi.game(n = 1000,p.or.m = .01,directed = FALSE,loops = FALSE)
#g<- simplify(g,remove.loops = TRUE,remove.multiple = TRUE)
#e_list= get.edgelist(g,names = FALSE)
#resultsnew= counts_in_egonetworks(e_list,step=2)

#require(netdisR)
#adj=get.adjacency(g,type="both",sparse=FALSE)
#netdiscounts=netdis_orcaegocounts(adj)
#rownames(netdiscounts)=NULL

#tail(resultsnew[,2:9])
#tail(netdiscounts[,2:9])

#identical(colSums(resultsnew[,2:9]),colSums(netdiscounts[,2:9],na.rm=TRUE) )
#########////



